@@ -10,8 +10,8 @@ which are available at :py:attr:`UNFCCCApiReader.annex_one_reader` and
 
 __author__ = """Mika Pflüger"""
 __email__ = "mika.pflueger@pik-potsdam.de"
-__version__ = "2.0.1"
+__version__ = "3.0.2"
 
-from .unfccc_di_api import UNFCCCApiReader, UNFCCCSingleCategoryApiReader
+from .unfccc_di_api import NoDataError, UNFCCCApiReader, UNFCCCSingleCategoryApiReader
 
-__all__ = ["UNFCCCApiReader", "UNFCCCSingleCategoryApiReader"]
+__all__ = ["UNFCCCApiReader", "UNFCCCSingleCategoryApiReader", "NoDataError"]
