@@ -11,7 +11,7 @@ citation = f"""Citation
 --------
 If you use this library and want to cite it, please cite it as:
 
-Mika Pflüger & Daniel Huppmann. ({new_date}).
+Mika Pflüger, Daniel Huppmann & Johannes Gütschow. ({new_date}).
 {new_title}.
 Zenodo. {new_link}
 """
